#ifndef bqtNewRadixSetHH
#define bqtNewRadixSetHH

#include "RadixTreeGMP-Vector.hh"

#include "bigint.hh"
#include "templates.hh"

template<unsigned RealSize>
class NewRadixSetGMP: private RadixTreeGMP<RealSize,0>
{
private:
    typedef RadixTreeGMP<RealSize,0> parent;
public:
    using parent::size;
    using parent::clear;
    inline bool find(BigInt& Key) const { return parent::find(Key); }
    inline bool add(BigInt& Key) { return parent::add(Key); }
};

template<unsigned RealSize, typename DataType>
class NewRadixMapGMP: private RadixTreeGMP<RealSize,sizeof(DataType)>
{
private:
    typedef RadixTreeGMP<RealSize,sizeof(DataType)> parent;
public:
    using parent::size;
    using parent::clear;
    inline bool find(BigInt& Key, DataType& value) const
    {
        return parent::find(Key, (unsigned char*)&value);
    }
    inline bool add(BigInt& Key, const DataType& value)
    {
        return parent::add(Key, (const unsigned char*)&value);
    }
    inline bool add_or_find(BigInt& Key, DataType& value)
    {
        return parent::add_or_find(Key, (unsigned char*)&value);
    }
    inline void ensure(BigInt& Key, const DataType& value)
    {
        parent::ensure(Key, (const unsigned char*)&value);
    }
};

template<typename KeyType, unsigned MaxValue, unsigned KeyLength>
class NewRadixEncoder
{
protected:
    static BigInt& Encode(const KeyType*const Key)
    {
        static BigInt key_enc;
        key_enc = 0;
        for(unsigned a=0; a<KeyLength; ++a)
        {
            if(a>0)
            {
                if(MaxValue==0xFFFFFFFFU)
                    key_enc <<= 32;
                else if(MaxValue==0xFFFFU)
                    key_enc <<= 16;
                else
                    key_enc *= (MaxValue+1);
            }
            key_enc += Key[a];
        }
        return key_enc;
    }
};

template<typename KeyType, unsigned MaxValue, unsigned KeyLength>
class NewRadixSet:
    private NewRadixSetGMP<BytesUsedBy<MaxValue+1,KeyLength>::MaxBytes>,
    private NewRadixEncoder<KeyType,MaxValue,KeyLength>
{
private:
    typedef NewRadixSetGMP<BytesUsedBy<MaxValue+1,KeyLength>::MaxBytes> parent;
    using NewRadixEncoder<KeyType,MaxValue,KeyLength>::Encode;
public:
    using parent::size;
    using parent::clear;
    inline bool find(const KeyType*const Key) const { return parent::find(Encode(Key)); }
    inline bool add(const KeyType*const Key) { return parent::add(Encode(Key)); }
};

template<typename KeyType, unsigned MaxValue, unsigned KeyLength, typename DataType>
class NewRadixMap:
    private NewRadixMapGMP<BytesUsedBy<MaxValue+1,KeyLength>::MaxBytes, DataType>,
    private NewRadixEncoder<KeyType,MaxValue,KeyLength>
{
private:
    typedef NewRadixMapGMP<BytesUsedBy<MaxValue+1,KeyLength>::MaxBytes, DataType> parent;
    using NewRadixEncoder<KeyType,MaxValue,KeyLength>::Encode;
public:
    using parent::size;
    using parent::clear;
    inline bool find(const KeyType*const Key, DataType& value) const
    {
        return parent::find(Encode(Key), value);
    }
    inline bool add(const KeyType*const Key, const DataType& value)
    {
        return parent::add(Encode(Key), value);
    }
    inline bool add_or_find(const KeyType*const Key, DataType& value)
    {
        return parent::add_or_find(Encode(Key), value);
    }
    inline void ensure(const KeyType*const Key, const DataType& value)
    {
        parent::ensure(Encode(Key), value);
    }
};

#endif
