#ifndef bqtBigIntHH
#define bqtBigIntHH

#include <gmp.h>

class BigInt
{
public:
    inline BigInt();
    inline BigInt(unsigned value);
    inline BigInt(const BigInt& b);
    ~BigInt();

    inline BigInt& operator= (unsigned v);
    inline BigInt& operator= (const BigInt& b);
    inline BigInt& operator<<= (unsigned v);

    inline BigInt& operator+= (unsigned v);
    inline BigInt& operator-= (unsigned v);
    inline BigInt& operator*= (unsigned v);
    inline BigInt& operator/= (unsigned v);

    inline BigInt& operator+= (const BigInt& v);
    inline BigInt& operator-= (const BigInt& v);
    inline BigInt& operator*= (const BigInt& v);
    inline BigInt& operator/= (const BigInt& v);
    
    inline const BigInt operator+ (const BigInt& v) const { BigInt result=*this; result += v; return result; }
    inline const BigInt operator- (const BigInt& v) const { BigInt result=*this; result -= v; return result; }
    inline const BigInt operator* (const BigInt& v) const { BigInt result=*this; result *= v; return result; }
    inline const BigInt operator/ (const BigInt& v) const { BigInt result=*this; result /= v; return result; }
    
    inline const BigInt operator+ (unsigned v) const { BigInt result=*this; result += v; return result; }
    inline const BigInt operator- (unsigned v) const { BigInt result=*this; result -= v; return result; }
    inline const BigInt operator* (unsigned v) const { BigInt result=*this; result *= v; return result; }
    inline const BigInt operator/ (unsigned v) const { BigInt result=*this; result /= v; return result; }
    
    inline operator bool() const;
    
    inline unsigned GetInt() const;
    inline double GetDouble() const;
    inline unsigned GetLog2Ceil() const;
    
    inline unsigned operator% (unsigned v) const;
    
    inline unsigned DivMod(BigInt& q, BigInt& r, unsigned v) const;
    inline void DivMod(BigInt& q, BigInt& r, const BigInt& v) const;

    inline unsigned DivMod2Exp(BigInt& q, BigInt& r, unsigned v) const;
    
    inline void Pow(unsigned b,unsigned e);
    
    inline void Choose(unsigned n,unsigned k);
    
    inline const mpz_t& GetMpz() const { return mpz; }
private:
    mpz_t mpz;
};

inline std::ostream& operator<< (std::ostream& o, const BigInt& i)
{
    return o << i.GetMpz();
}

#include "bigint.cc"

#endif
