#include "filewindow.hh"
#include <cstdlib>

template<typename DataType>
class MemoryStorage
{
private:
    unsigned char* buf;
    off64_t Cap;
    off64_t Size;
private:
    void Ensure(unsigned endpos)
    {
        /* Ensure that the given amount of data exists */
        if(endpos > Size) Size = endpos;
        if(endpos > Cap)
        {
            unsigned align = 0x100000;
            Cap = (endpos + align-1) & ~(align-1);
            
            /* Note: realloc(NULL,..) is equivalent to malloc(...) */
            buf = (unsigned char*)std::realloc(buf, Cap);
        }
    }
public:
    MemoryStorage(): buf(0), Cap(0), Size(0)
    {
    }
    ~MemoryStorage() { if(buf) std::free(buf); }
    
    inline bool empty() const   { return !Size; }
    inline off64_t size() const { return Size; }
    inline void clear()
    {
        Size=0;
        //Cap=0;
        //if(buf) { std::free(buf); buf=0; }
    }

    /* Pointers returned by GetWritePtr() and GetReadPtr()
     * are guaranteed to be valid only until the next call
     * of either function or clear().
     */
    DataType* GetWritePtr(unsigned pos, unsigned wsize=1)
    {
        pos*=sizeof(DataType); wsize*=sizeof(DataType);
        Ensure(pos+wsize);
        return (DataType*)(buf+pos);
    }
    const DataType* GetReadPtr(unsigned pos, unsigned wsize=1)
    {
        pos*=sizeof(DataType); wsize*=sizeof(DataType);
        Ensure(pos+wsize);
        return (const DataType*)(buf+pos);
    }
private:
    /* Copying is not allowed.
     * However, this is not a singleton class.
     * Multiple instances can exist. Each has their own address space.
     */
    MemoryStorage(const MemoryStorage&);
    void operator=(const MemoryStorage&);
};

template<typename DataType>
class DiskStorage
  : private TempFileBuffer<1048576*16,8,DataType>
{
private:
    typedef TempFileBuffer<1048576*16,8,DataType> parent;
    off64_t Size;
public:
    DiskStorage(): Size(0) { }
    
    inline bool empty() const   { return !Size; }
    inline off64_t size() const { return Size; }
    inline void clear() { parent::clear(); Size=0; }

    /* Pointers returned by GetWritePtr() and GetReadPtr()
     * are guaranteed to be valid only until the next call
     * of either function or clear().
     */
    DataType* GetWritePtr(off64_t index, unsigned amount=1)
    {
        off64_t end = index+amount; if(end > Size)Size=end;
        return parent::GetElemWindow(index,amount);
    }
    const DataType* GetReadPtr(off64_t index, unsigned amount=1)
    {
        off64_t end = index+amount; if(end > Size)Size=end;
        return parent::GetElemWindow(index,amount);
    }
private:
    /* Copying is not allowed.
     * However, this is not a singleton class.
     * Multiple instances can exist. Each has their own address space.
     */
    DiskStorage(const DiskStorage&);
    void operator=(const DiskStorage&);
};

